/*************************************************************************************************************************************************/
/*!     DAC161.c
*
*       This code is designed to perform standard command and control operations on the DAC161 over a SPI bus. Functions exist to setup, configure,
*       and control the DAC161.
*
*       The software is specifically written to execute on an MSP430G2413 on the SATxxxx board.
*
*       October 2013
*
*       \note that the functions in this file are not re-entrant. It is the user's responsibility to assure that these functions
*       are not called until the previous function has completed.
*/
/**************************************************************************************************************************************************
*       Copyright  2013 Texas Instruments Incorporated - http://www.ti.com/                                                                      *
***************************************************************************************************************************************************
*  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met: *
*                                                                                                                                                 *
*    Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.                 *
*                                                                                                                                                 *
*    Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the        *
*    documentation and/or other materials provided with the distribution.                                                                         *
*                                                                                                                                                 *
*    Neither the name of Texas Instruments Incorporated nor the names of its contributors may be used to endorse or promote products derived      *
*    from this software without specific prior written permission.                                                                                *
*                                                                                                                                                 *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT          *
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     *
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT         *
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY    *
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE      *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                                                           *
**************************************************************************************************************************************************/
#include "usci_spi.h"
#include "DAC161.h"

static unsigned char SpiHandle;
static unsigned char RcvData[3];

/*************************************************************************************************************************************************
*  DAC161_Write_Regs
**************************************************************************************************************************************************/
/*!
* @brief Writes registers on the DAC161.
*
* This function will execute a write register command to the DAC161. This function can be used to update one or more registers on the DAC161.
* No error checking is performed, so it is the user's responsibility to make sure they do not attempt to write past the end of the DAC161 registers.
*
* @param[out]   *writeValues    Pointer to the list of 8 bit register values to place in the DAC161
* @param[in]     startReg       Address of the first register to write
* @param[in]     length         Number of registers to write.
*
* @return  None
*
**************************************************************************************************************************************************/
void DAC161_Write_Regs (unsigned short *writeValues, unsigned char startReg, unsigned char lengthBytes)
{
    unsigned char outData[3];

    outData[0] = DAC161_SPI_WRITE_CMD(startReg);

    // Switch Endianess
    outData[1] = *writeValues >> 8;
    outData[2] = *writeValues & 0xff;

    Change_SPI_Phase (SPI_CAPT_ON_FIRST_EDGE);
    SPI_Write (SpiHandle, outData, RcvData, lengthBytes+1);    // Add 1 to length for command byte

}

/*************************************************************************************************************************************************
*  DAC161_Read_Regs
**************************************************************************************************************************************************/
/*!
* @brief Reads registers on the DAC161.
*
* This function will execute a read register command to the DAC161 and return the resultant data. This function can be used to read one or more
* registers from the DAC161. No error checking is performed, so it is the user's responsibility to make sure they do not attempt to read past
* the end of the DAC161 registers.
*
* @param[out]   *readValues     Pointer to place the 8 bit register values from the DAC161
* @param[in]     startReg       Address of the first register to read
* @param[in]     length         Number of registers to read.
*
* @return  None
*
**************************************************************************************************************************************************/
void DAC161_Read_Regs (unsigned char *readValues, unsigned char startReg, unsigned char lengthBytes)
{
    unsigned char outData[3] = {0x55, 0x55, 0x55};

    Change_SPI_Phase (SPI_CAPT_ON_FIRST_EDGE);

    // Sets the address for the read command
    outData[0] = DAC161_SPI_READ_CMD(startReg);
    SPI_Write (SpiHandle, outData, readValues, lengthBytes+1);    // Add 1 to length for command byte

    // Performs actual read of previous address
    outData[0] = DAC161_NOP_REG;
    SPI_Write (SpiHandle, outData, readValues, lengthBytes+1);    // Add 1 to length for command byte

}

/*************************************************************************************************************************************************
*  Setup_DAC161
**************************************************************************************************************************************************/
/*!
* @brief Performs the setup of the DAC161.
*
* This function will configure the DAC161.
*
* @param[in]     ste_port        The MSP430 port that contains the CS or STE pin that is connected to the DAC161. (PORT_1, PORT_2, ...)
* @param[in]     ste_pin         The MSP430 pin that contains the CS or STE pin that is connected to the DAC161.  (BIT0, BIT1, ...)
* @param[in]     errConfig       Error Configuration for the DAC161 (DAC161_STD_ERR_CONFIG)
* @param[in]     errLow_uA       Output level (micro Amps) for a Low Error
* @param[in]     errHigh_uA      Output level (micro Amps) for a High Error
*
* @return  None
*
**************************************************************************************************************************************************/
void Setup_DAC161  (Port_t ste_port, unsigned char ste_pin, unsigned short errConfig, unsigned short errLow_uA, unsigned short errHigh_uA)
{
    unsigned short errValue;

    SpiHandle = RegisterSpiDevice (ste_port, ste_pin);

    DAC161_Write_Regs (&errConfig, DAC161_ERR_CONFIG_REG, 2);

    errValue = DAC161_CONVERT_ERR_VALUE(errLow_uA) & 0x7f00;
    DAC161_Write_Regs (&errValue, DAC161_ERR_LOW_REG, 2);

    errValue = DAC161_CONVERT_ERR_VALUE(errHigh_uA) & 0xff00;
    if (errValue < 0x80)
        errValue = 0x80;
    DAC161_Write_Regs (&errValue, DAC161_ERR_HIGH_REG, 2);

}


/*************************************************************************************************************************************************
*  DAC161_Reset
**************************************************************************************************************************************************/
/*!
* @brief Sends a Reset Command to the DAC161.
*
* This function sends a Reset command to the DAC161 on the SPI bus.
*
* @return  None
*
**************************************************************************************************************************************************/
void DAC161_Reset (void)
{
    unsigned short resetCode = DAC161_RESET_VALUE;

    DAC161_Write_Regs (&resetCode, DAC161_RESET_REG, 2);
    DAC161_Write_Regs (&resetCode, DAC161_NOP_REG, 2);          // Nop required after reset

}

/*************************************************************************************************************************************************
*  DAC161_Nop
**************************************************************************************************************************************************/
/*!
* @brief Sends a Nop Command to the DAC161.
*
* This function sends a Nop command to the DAC161 on the SPI bus. The DAC161 will timeout and move into an error state if it does not receive
* regular commands for the SPI master. This command can be used to notify the DAC161 that the system is still operational, but no change to the
* DAC161 is desired.
*
* @return  None
*
**************************************************************************************************************************************************/
void DAC161_Nop (void)
{
    unsigned short deadCode = 0xdead;

    DAC161_Write_Regs (&deadCode, DAC161_NOP_REG, 2);

}

/*************************************************************************************************************************************************
*  DAC161_Set_Out_Value
**************************************************************************************************************************************************/
/*!
* @brief Sets the output current from the DAC161.
*
* The DAC161 is designed to be used in a 4-20mAmp system. Data is communicated by changing the current output from the DAC161. This function sets
* the desired output current.
*
* @param[in]     uAmps         Value in uAmps to output from the DAC161
*
* @return  None
*
**************************************************************************************************************************************************/
void DAC161_Set_Out_Value (unsigned long uAmps)
{
    unsigned short value = DAC161_OUTPUT_CONVERT(uAmps);

    DAC161_Write_Regs (&value, DAC161_DACCODE_REG, 2);

}

/*************************************************************************************************************************************************
*  DAC161_Read_Status
**************************************************************************************************************************************************/
/*!
* @brief Reads thE status register from the DAC161.
*
* This function returns the current value in the status register of the DAC161
*
* @return  DAC161 Status     (DAC161_FERR_STS, DAC161_SPI_TIMEOUT_ERR, DAC161_LOOP_STS, DAC161_CURR_LOOP_STS)
**************************************************************************************************************************************************/
unsigned char DAC161_Read_Status (void)
{
    unsigned char returnValue[3];

    DAC161_Read_Regs (returnValue, DAC161_STATUS_REG, 2);

    return (returnValue[2]);
}
