/*************************************************************************************************************************************************/
/*!     RTD_Math.c
*
*       This code is designed to perform fixed point math calculations needed to convert the conversion value returned from the ADS1220 (or other
*       24 bit ADC) into an accurate temperature reading.
*
*       The software is specifically written to execute on an MSP430G2413 on the SATxxxx board.
*
*       October 2013
*
*       \note that the functions in this file are not re-entrant. It is the user's responsibility to assure that these functions
*       are not called until the previous function has completed.
*/
/**************************************************************************************************************************************************
*       Copyright  2013 Texas Instruments Incorporated - http://www.ti.com/                                                                      *
***************************************************************************************************************************************************
*  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met: *
*                                                                                                                                                 *
*    Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.                 *
*                                                                                                                                                 *
*    Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the        *
*    documentation and/or other materials provided with the distribution.                                                                         *
*                                                                                                                                                 *
*    Neither the name of Texas Instruments Incorporated nor the names of its contributors may be used to endorse or promote products derived      *
*    from this software without specific prior written permission.                                                                                *
*                                                                                                                                                 *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT          *
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     *
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT         *
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY    *
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE      *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                                                           *
**************************************************************************************************************************************************/
#include "RTD_Math.h"
#include "ADS1220.h"
#include <math.h>

extern float Rref;

// This particular table has measurements at every degree C from -200 to 851 inclusive (1052 points).
// It is used to perform a linear interpolation between two points that surround the measurement. In this case, those points are 1 degree apart.
//
// Other tables can be used with more or less points, at the impact of changing the accuracy of the measurement. These is no requirement for the measurements to be evenly
// spaced. The table can have fewer points at areas that are nearly linear, and more points at areas that are less linear with no change required to the algorithm.
//
//
const float Points[] =
{
        18.520, 18.952, 19.384, 19.815, 20.247, 20.677, 21.108, 21.538, 21.967, 22.397,
        22.825, 23.254, 23.682, 24.110, 24.538, 24.965, 25.392, 25.819, 26.245, 26.671,
        27.096, 27.522, 27.947, 28.371, 28.796, 29.220, 29.643, 30.067, 30.490, 30.913,
        31.335, 31.757, 32.179, 32.601, 33.022, 33.443, 33.864, 34.284, 34.704, 35.124,
        35.543, 35.963, 36.382, 36.800, 37.219, 37.637, 38.055, 38.472, 38.889, 39.306,
        39.723, 40.140, 40.556, 40.972, 41.388, 41.803, 42.218, 42.633, 43.048, 43.462,
        43.876, 44.290, 44.704, 45.117, 45.531, 45.944, 46.356, 46.769, 47.181, 47.593,
        48.005, 48.416, 48.828, 49.239, 49.649, 50.060, 50.470, 50.881, 51.291, 51.700,
        52.110, 52.519, 52.928, 53.337, 53.746, 54.154, 54.562, 54.970, 55.378, 55.786,
        56.193, 56.600, 57.007, 57.414, 57.821, 58.227, 58.633, 59.039, 59.445, 59.850,
        60.256, 60.661, 61.066, 61.471, 61.876, 62.280, 62.684, 63.088, 63.492, 63.896,
        64.300, 64.703, 65.106, 65.509, 65.912, 66.315, 66.717, 67.120, 67.522, 67.924,
        68.325, 68.727, 69.129, 69.530, 69.931, 70.332, 70.733, 71.134, 71.534, 71.934,
        72.335, 72.735, 73.134, 73.534, 73.934, 74.333, 74.732, 75.131, 75.530, 75.929,
        76.328, 76.726, 77.125, 77.523, 77.921, 78.319, 78.717, 79.114, 79.512, 79.909,
        80.306, 80.703, 81.100, 81.497, 81.894, 82.290, 82.687, 83.083, 83.479, 83.875,
        84.271, 84.666, 85.062, 85.457, 85.853, 86.248, 86.643, 87.038, 87.432, 87.827,
        88.222, 88.616, 89.010, 89.404, 89.798, 90.192, 90.586, 90.980, 91.373, 91.767,
        92.160, 92.553, 92.946, 93.339, 93.732, 94.124, 94.517, 94.909, 95.302, 95.694,
        96.086, 96.478, 96.870, 97.261, 97.653, 98.044, 98.436, 98.827, 99.218, 99.609,
        100.000, 100.391, 100.781, 101.172, 101.562, 101.953, 102.343, 102.733, 103.123, 103.513,
        103.903, 104.292, 104.682, 105.071, 105.460, 105.849, 106.238, 106.627, 107.016, 107.405,
        107.794, 108.182, 108.570, 108.959, 109.347, 109.735, 110.123, 110.510, 110.898, 111.286,
        111.673, 112.060, 112.447, 112.835, 113.221, 113.608, 113.995, 114.382, 114.768, 115.155,
        115.541, 115.927, 116.313, 116.699, 117.085, 117.470, 117.856, 118.241, 118.627, 119.012,
        119.397, 119.782, 120.167, 120.552, 120.936, 121.321, 121.705, 122.090, 122.474, 122.858,
        123.242, 123.626, 124.009, 124.393, 124.777, 125.160, 125.543, 125.926, 126.309, 126.692,
        127.075, 127.458, 127.840, 128.223, 128.605, 128.987, 129.370, 129.752, 130.133, 130.515,
        130.897, 131.278, 131.660, 132.041, 132.422, 132.803, 133.184, 133.565, 133.946, 134.326,
        134.707, 135.087, 135.468, 135.848, 136.228, 136.608, 136.987, 137.367, 137.747, 138.126,
        138.506, 138.885, 139.264, 139.643, 140.022, 140.400, 140.779, 141.158, 141.536, 141.914,
        142.293, 142.671, 143.049, 143.426, 143.804, 144.182, 144.559, 144.937, 145.314, 145.691,
        146.068, 146.445, 146.822, 147.198, 147.575, 147.951, 148.328, 148.704, 149.080, 149.456,
        149.832, 150.208, 150.583, 150.959, 151.334, 151.710, 152.085, 152.460, 152.835, 153.210,
        153.584, 153.959, 154.333, 154.708, 155.082, 155.456, 155.830, 156.204, 156.578, 156.952,
        157.325, 157.699, 158.072, 158.445, 158.818, 159.191, 159.564, 159.937, 160.309, 160.682,
        161.054, 161.427, 161.799, 162.171, 162.543, 162.915, 163.286, 163.658, 164.030, 164.401,
        164.772, 165.143, 165.514, 165.885, 166.256, 166.627, 166.997, 167.368, 167.738, 168.108,
        168.478, 168.848, 169.218, 169.588, 169.958, 170.327, 170.696, 171.066, 171.435, 171.804,
        172.173, 172.542, 172.910, 173.279, 173.648, 174.016, 174.384, 174.752, 175.120, 175.488,
        175.856, 176.224, 176.591, 176.959, 177.326, 177.693, 178.060, 178.427, 178.794, 179.161,
        179.528, 179.894, 180.260, 180.627, 180.993, 181.359, 181.725, 182.091, 182.456, 182.822,
        183.188, 183.553, 183.918, 184.283, 184.648, 185.013, 185.378, 185.743, 186.107, 186.472,
        186.836, 187.200, 187.564, 187.928, 188.292, 188.656, 189.019, 189.383, 189.746, 190.110,
        190.473, 190.836, 191.199, 191.562, 191.924, 192.287, 192.649, 193.012, 193.374, 193.736,
        194.098, 194.460, 194.822, 195.183, 195.545, 195.906, 196.268, 196.629, 196.990, 197.351,
        197.712, 198.073, 198.433, 198.794, 199.154, 199.514, 199.875, 200.235, 200.595, 200.954,
        201.314, 201.674, 202.033, 202.393, 202.752, 203.111, 203.470, 203.829, 204.188, 204.546,
        204.905, 205.263, 205.622, 205.980, 206.338, 206.696, 207.054, 207.411, 207.769, 208.127,
        208.484, 208.841, 209.198, 209.555, 209.912, 210.269, 210.626, 210.982, 211.339, 211.695,
        212.052, 212.408, 212.764, 213.120, 213.475, 213.831, 214.187, 214.542, 214.897, 215.252,
        215.608, 215.962, 216.317, 216.672, 217.027, 217.381, 217.736, 218.090, 218.444, 218.798,
        219.152, 219.506, 219.860, 220.213, 220.567, 220.920, 221.273, 221.626, 221.979, 222.332,
        222.685, 223.038, 223.390, 223.743, 224.095, 224.447, 224.799, 225.151, 225.503, 225.855,
        226.206, 226.558, 226.909, 227.260, 227.612, 227.963, 228.314, 228.664, 229.015, 229.366,
        229.716, 230.066, 230.417, 230.767, 231.117, 231.467, 231.816, 232.166, 232.516, 232.865,
        233.214, 233.564, 233.913, 234.262, 234.610, 234.959, 235.308, 235.656, 236.005, 236.353,
        236.701, 237.049, 237.397, 237.745, 238.093, 238.440, 238.788, 239.135, 239.482, 239.829,
        240.176, 240.523, 240.870, 241.217, 241.563, 241.910, 242.256, 242.602, 242.948, 243.294,
        243.640, 243.986, 244.331, 244.677, 245.022, 245.367, 245.713, 246.058, 246.403, 246.747,
        247.092, 247.437, 247.781, 248.125, 248.470, 248.814, 249.158, 249.502, 249.845, 250.189,
        250.533, 250.876, 251.219, 251.562, 251.906, 252.248, 252.591, 252.934, 253.277, 253.619,
        253.962, 254.304, 254.646, 254.988, 255.330, 255.672, 256.013, 256.355, 256.696, 257.038,
        257.379, 257.720, 258.061, 258.402, 258.743, 259.083, 259.424, 259.764, 260.105, 260.445,
        260.785, 261.125, 261.465, 261.804, 262.144, 262.483, 262.823, 263.162, 263.501, 263.840,
        264.179, 264.518, 264.857, 265.195, 265.534, 265.872, 266.210, 266.548, 266.886, 267.224,
        267.562, 267.900, 268.237, 268.574, 268.912, 269.249, 269.586, 269.923, 270.260, 270.597,
        270.933, 271.270, 271.606, 271.942, 272.278, 272.614, 272.950, 273.286, 273.622, 273.957,
        274.293, 274.628, 274.963, 275.298, 275.633, 275.968, 276.303, 276.638, 276.972, 277.307,
        277.641, 277.975, 278.309, 278.643, 278.977, 279.311, 279.644, 279.978, 280.311, 280.644,
        280.978, 281.311, 281.643, 281.976, 282.309, 282.641, 282.974, 283.306, 283.638, 283.971,
        284.303, 284.634, 284.966, 285.298, 285.629, 285.961, 286.292, 286.623, 286.954, 287.285,
        287.616, 287.947, 288.277, 288.608, 288.938, 289.268, 289.599, 289.929, 290.258, 290.588,
        290.918, 291.247, 291.577, 291.906, 292.235, 292.565, 292.894, 293.222, 293.551, 293.880,
        294.208, 294.537, 294.865, 295.193, 295.521, 295.849, 296.177, 296.505, 296.832, 297.160,
        297.487, 297.814, 298.142, 298.469, 298.795, 299.122, 299.449, 299.775, 300.102, 300.428,
        300.754, 301.080, 301.406, 301.732, 302.058, 302.384, 302.709, 303.035, 303.360, 303.685,
        304.010, 304.335, 304.660, 304.985, 305.309, 305.634, 305.958, 306.282, 306.606, 306.930,
        307.254, 307.578, 307.902, 308.225, 308.549, 308.872, 309.195, 309.518, 309.841, 310.164,
        310.487, 310.810, 311.132, 311.454, 311.777, 312.099, 312.421, 312.743, 313.065, 313.386,
        313.708, 314.029, 314.351, 314.672, 314.993, 315.314, 315.635, 315.956, 316.277, 316.597,
        316.918, 317.238, 317.558, 317.878, 318.198, 318.518, 318.838, 319.157, 319.477, 319.796,
        320.116, 320.435, 320.754, 321.073, 321.391, 321.710, 322.029, 322.347, 322.666, 322.984,
        323.302, 323.620, 323.938, 324.256, 324.573, 324.891, 325.208, 325.526, 325.843, 326.160,
        326.477, 326.794, 327.110, 327.427, 327.744, 328.060, 328.376, 328.692, 329.008, 329.324,
        329.640, 329.956, 330.271, 330.587, 330.902, 331.217, 331.533, 331.848, 332.162, 332.477,
        332.792, 333.106, 333.421, 333.735, 334.049, 334.363, 334.677, 334.991, 335.305, 335.619,
        335.932, 336.246, 336.559, 336.872, 337.185, 337.498, 337.811, 338.123, 338.436, 338.748,
        339.061, 339.373, 339.685, 339.997, 340.309, 340.621, 340.932, 341.244, 341.555, 341.867,
        342.178, 342.489, 342.800, 343.111, 343.422, 343.732, 344.043, 344.353, 344.663, 344.973,
        345.284, 345.593, 345.903, 346.213, 346.522, 346.832, 347.141, 347.451, 347.760, 348.069,
        348.378, 348.686, 348.995, 349.303, 349.612, 349.920, 350.228, 350.536, 350.844, 351.152,
        351.460, 351.768, 352.075, 352.382, 352.690, 352.997, 353.304, 353.611, 353.918, 354.224,
        354.531, 354.837, 355.144, 355.450, 355.756, 356.062, 356.368, 356.674, 356.979, 357.285,
        357.590, 357.896, 358.201, 358.506, 358.811, 359.116, 359.420, 359.725, 360.029, 360.334,
        360.638, 360.942, 361.246, 361.550, 361.854, 362.158, 362.461, 362.765, 363.068, 363.371,
        363.674, 363.977, 364.280, 364.583, 364.886, 365.188, 365.491, 365.793, 366.095, 366.397,
        366.699, 367.001, 367.303, 367.604, 367.906, 368.207, 368.508, 368.810, 369.111, 369.412,
        369.712, 370.013, 370.314, 370.614, 370.914, 371.215, 371.515, 371.815, 372.115, 372.414,
        372.714, 373.013, 373.313, 373.612, 373.911, 374.210, 374.509, 374.808, 375.107, 375.406,
        375.704, 376.002, 376.301, 376.599, 376.897, 377.195, 377.493, 377.790, 378.088, 378.385,
        378.683, 378.980, 379.277, 379.574, 379.871, 380.167, 380.464, 380.761, 381.057, 381.353,
        381.650, 381.946, 382.242, 382.537, 382.833, 383.129, 383.424, 383.720, 384.015, 384.310,
        384.605, 384.900, 385.195, 385.489, 385.784, 386.078, 386.373, 386.667, 386.961, 387.255,
        387.549, 387.843, 388.136, 388.430, 388.723, 389.016, 389.310, 389.603, 389.896, 390.188,
        390.481, 390.774
};

/*************************************************************************************************************************************************
*  interpolateTemperatureValue
**************************************************************************************************************************************************/
/*!
* @brief Returns the temperature in millidegrees that corresponds to the RTD code value returned from the ADS1220.
*
* The function converts the code value returned from the RTRD reading using the ADS1220 and returns the temperature value in milldegrees C.
*
* @param[in]   code                24 bit value provided from the ADS1220 from the RTD conversion
*
* @return  temperature             Temperature in millidegrees C.
*
**************************************************************************************************************************************************/
float interpolateTemperatureValue (long code)
{
    static float interimValue;
    static float targetValue;
    static float r_value;
    unsigned short index;

    r_value = calculateRValue (code);
    index = findInterpolationIndex(r_value);

    // Interim value will be a fraction less than 1
    interimValue = (r_value - Points[index]) / (Points[index+1] - Points[index]);

    // The algorithm requires the table to be one value for every degree. Since the table is regular, the
    //  interimValue will be the decimal portion of the temperature measurement. The Index show how deep
    //  we are into the table with each step being one degree apart. If we add value in degree C of the
    //  first measurement in the table we can convert into temperature
    targetValue = interimValue  + ((long)index + MIN_TEMP_MEAS);        // Convert index out of unsigned variable to allow for negative number

    return (targetValue);  // Note this is in Degrees C
}

/*************************************************************************************************************************************************
*  findInterpolationIndex
**************************************************************************************************************************************************/
/*!
* @brief Searches the Points table to locate the index where the input R value lands in between.
*
* The function requires a Points table (above) containing temperatures (in millidegrees), and R values. The R Values must be monotomically increasing.
* This function will return the index of the lower of the two points surrounding the input R value. These points can then be used to perform
* linear interpretation to get the approximate temperature.
*
* @param[in]   r_value             The r_value from the RTD return
*
* @return  index value             The index value pointing to the lower point that is closest to the input r_value.
*
* @note  Since the Interpolation table has one entry for degree C, starting at -200 C, you cna subtract 200 from the index to get the associated
*         temperature in degrees
**************************************************************************************************************************************************/
short findInterpolationIndex (float r_value)
{
    unsigned long i;

    for (i=0; i< NUM_MEAS_POINTS; i++)
    {
        if (Points[i] > r_value)
            return (i-1);
    }

    return (-1);
}


/*************************************************************************************************************************************************
*  calculateRValue
**************************************************************************************************************************************************/
/*!
* @brief This function converts the code returned from the conversion of the RTD in the ADS1220 into an R Value.
*
* The algorithm to perform the conversion goes as follows :
*         rtd = (code * Rref * 2.0) / (PgaGainLevel * (2^23 - 1));
*
*
* @param[in]   code                24 bit value provided from the ADS1220 from the RTD conversion
*
* @return  r_value                 The r_value from the RTD return
*
**************************************************************************************************************************************************/
float calculateRValue (long code)
{
    float rtd;            // Must be volatile to force the order of operations below


    rtd = (code * Rref * 2.0) / (PgaGainLevel * 8388607.0);

    return (rtd);
}



