/*************************************************************************************************************************************************/
/*!     Temperature Transmitter Code
*
*       This code is designed to implement a Temperature Transmitter application highlighting the ADS1220 to receive data from an RTD Temperature
*       probe and send out the temperature reading on a 4-20mA signal using the DAC161.
*
*       The software is specifically written to execute on an MSP430G2413 on the SATxxxx board.
*
*       October 2013
*
*       \note that the functions in this file are not re-entrant. It is the user's responsibility to assure that these functions
*       are not called until the previous function has completed.
*/
/**************************************************************************************************************************************************
*       Copyright  2013 Texas Instruments Incorporated - http://www.ti.com/                                                                      *
***************************************************************************************************************************************************
*  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met: *
*                                                                                                                                                 *
*    Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.                 *
*                                                                                                                                                 *
*    Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the        *
*    documentation and/or other materials provided with the distribution.                                                                         *
*                                                                                                                                                 *
*    Neither the name of Texas Instruments Incorporated nor the names of its contributors may be used to endorse or promote products derived      *
*    from this software without specific prior written permission.                                                                                *
*                                                                                                                                                 *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT          *
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     *
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT         *
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY    *
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE      *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                                                           *
**************************************************************************************************************************************************/

#include <msp430.h>
#include "usci_digital_io.h"
#include "usci_spi.h"
#include "usci_uart.h"
#include "TempTransmitter.h"
#include "ADS1220.h"
#include "DAC161.h"
#include "RTD_Math.h"

#define SAMPLES_PER_DAC_ADJUSTMENT       16
#define TERMINAL_OUTPUT    1

float Rref = 3240.0;

// Setting the DacErrorCorrection to 1.0 and turning on the DAC_TEST will allow the user to measure the
//  output of the DAC161 at 2mA intervals to determine the DacErrorCorrection for there specific device.
//#define DAC_TEST 1
float DacErrorCorrection = 0.99945;

// The Gain Calibration is intended to be performed once for any system. The results will be stored in the
//  InfoC memory. To perform GainCalibration, a known precision resistor is placed into the spot for the RTD
//  and a series of measurements are taken. The measured value is compared to the expected result in the
//  MeasuredGainCodeValue variable below. The GainCalibration value is then calculated and stored in memory.

// To prevent the Calibration from being overwritten, the calibration only occurs when the user hits the
// top button AND the value in the InfoC memory is the erased memory value of 0xffffffff. If the user wishes to
// recompute the calibration, the JTAG must be configured to erase the Info Memory, or the user must change the
// value in the InfoC memory to 0xffffffff.

// To calculate expected result, take the expected answer from the precision multimeter reading and
//   Code = RTDmeas * PgaGainLevel * (2^23 - 1) / (Rref * 2)
float MeasuredGainCodeValue = 7868944.1883;



unsigned char StartConversion;
unsigned char ReadConversionData = 0;
unsigned char UART_Processing;
unsigned char DAC_Error_Processing;
unsigned char PrintFlag;
unsigned char GainCal = 0;

unsigned char DacIncrement = 1;              // Testing variable to allow the button to enable/disable DAC processing

extern long RawVoltage;
extern long CalVoltage;
extern long GainVoltage;
extern long GainCalibrateValue;

#pragma DATA_SECTION(FlashGainCorrection, ".infoC")
float FlashGainCorrection;

/*************************************************************************************************************************************************
*  Setup_Timer
**************************************************************************************************************************************************/
/*!
* @brief Initialize the Timer0 hardware to provide a specific tick interval
*
* This function is called once to configure the Timer0 hardware settings to provide a clock based on the SMCLK with a divider of 1. There will be
* a continuous interrupt generated based on the number of clocks cycles provided to this function as a  parameter.
*
* The user will be responsible for developing matching code in the TIMER0 A1 interrupt to receive the timer events.
*
* As an example
*    #pragma vector = TIMER0_A1_VECTOR
*    __interrupt void TIMER0_A1_ISR (void)
*    {
*        static unsigned char count = 0;
*
*        TA0CTL &= ~TAIFG;
*
*        // Start Conversion every 4th interrupt
*        if ((++count % 4) == 0)
*        {
*            StartConversion = 1;
*        }
*
*        __bic_SR_register_on_exit(CPUOFF);      // Exit LPM0
*    }
*
* @param[in]   timeSet             Number of SMCLK cycles to generate a timer interrupt
*
* @return  none
**************************************************************************************************************************************************/
void Setup_Timer(unsigned int timeSet)
{
    /***************************************************
    * Timer Configuration
    ***************************************************/
    TA0CCR0 = timeSet;
    TA0CTL = ((TASSEL_2) | (ID_3) | (MC_1) | (TAIE));
}


/*************************************************************************************************************************************************
*  Setup_IO
**************************************************************************************************************************************************/
/*!
* @brief Initialize the GPIO pins on the MSP430G2413 for use by the Temperature Transmitter
*
* This function is called once to configure the GPIO pins on the system.
*
* @return  none
**************************************************************************************************************************************************/
void Setup_IO (void)
{
    P1OUT |=  ADS_CS_N;                     // Active Low
    P1DIR |=  ADS_CS_N;
    P1IES |=  USR_USER_INTERRUPT_N;         // High to Low Edge
    P1IFG &= ~USR_USER_INTERRUPT_N;         // Clear any previous interrupt
    P1IE  |=  USR_USER_INTERRUPT_N;

    P2OUT &= ~(ERR_LVL);                    // Initially off
    P2OUT |=  (VCC_ISO_UART_EN);            // Set the isolation for UART
    P2DIR |=  (ERR_LVL | VCC_ISO_UART_EN);
    P2DIR &= ~(DAC_ERROR_N | ADS_DRDY_N | USR_GAIN_CAL);   // Input Pins
    P2IES |=  (DAC_ERROR_N | ADS_DRDY_N | USR_GAIN_CAL);   // High to Low Edge
    P2IFG &= ~(DAC_ERROR_N | ADS_DRDY_N | USR_GAIN_CAL);   // Clear any pending interrupts
    P2IE  |=  (DAC_ERROR_N | ADS_DRDY_N | USR_GAIN_CAL);   // Enable Interrupts

    P3OUT |= DAC_CS_N;                      // Active Low, so set inactive
    P3DIR |= DAC_CS_N;
}



/*************************************************************************************************************************************************
*  ConvertTemp2uAmps
**************************************************************************************************************************************************/
/*!
* @brief Converts a temperature in the range of -200 to +850 degrees C into a 4-20mAmp value.
*
* This function is called to convert the temperature (scaled into degrees C * 1000) into an Amp setting to send on the 4-20mA line.
* -200 degrees will correspond to 4.000 mA. +850 degrees C will correspond to 20mA.
*
* @param[in]      temperature  - value in degrees C * 1000; from -200000 to +850000
*
* @return  uAmp value       Input temperature converted in MicroAmps (mA * 1000).
**************************************************************************************************************************************************/
unsigned long ConvertTemp2uAmps (long temperature)
{
    float uAmps;

    // Here is where we need to convert the 3 bytes received from the ADS1220 into a uAmp value that will be placed into the DAC
    // -200 to +850 C measured in 1000 * C is  -200000 to 850000 or a range of 1050000 1000*C.
    // Add 200000 to the temperature to convert it into a range between 0 and 1050000. Divide by 1050000 to get the percentage of the temperature scale.
    //
    // 4 to 20mA output is measured in uAmps and therefore is 4000 to 20000 uAms or a range of 16000 uAmps.
    // Multiply the results above by 16000 to get the percentage on the uAmp scale that ranges from 0 - 16000.
    // Adjust the results so that -200 degrees C (which comes out to 0 uAmps) is represented by 4000 uAmps by adding 4000
    //
    // uAmps = ((temperature + 200000) * 16000 / 1050000) + 4000  --  Divide top and bottom by 1000 and we get
    //         ((temperature + 200000) * 16 / 1050) + 4000
    //

    uAmps = (((temperature + 200000) * 16.0) / 1050.0) + 4000.0;
    return ((unsigned long)uAmps);
}


/*************************************************************************************************************************************************
*  write_SegC
**************************************************************************************************************************************************/
/*!
* @brief Writes a floating point value into flash memory at the start of infoC
*
* This function is called to write a value into the infoC memory block of the MSP430. We specifically use it to store the ADS1220 Gain Correction.
*
* @param[in]      value - The floating point value that will be written into flash memory at the start of the infoC block
*
* @return  None
**************************************************************************************************************************************************/
void write_SegC (float value)
{
  char *flash_ptr;                          // Flash pointer
  unsigned int i;
  char *value_ptr = (char *)&value;

  flash_ptr = (char *) 0x1040;              // Initialize Flash pointer
  FCTL1 = FWKEY + ERASE;                    // Set Erase bit
  FCTL3 = FWKEY;                            // Clear Lock bit
  *flash_ptr = 0;                           // Dummy write to erase Flash segment

  FCTL1 = FWKEY + WRT;                      // Set WRT bit for write operation

  for (i=0; i<sizeof(value); i++)
  {
    *flash_ptr++ = *value_ptr++;                   // Write value to flash
  }

  FCTL1 = FWKEY;                            // Clear WRT bit
  FCTL3 = FWKEY + LOCK;                     // Set LOCK bit
}


void main (void)
{

volatile static unsigned char tempData[3];
unsigned long dacValueForTemp_uA = 20000;
unsigned long uncorrectedDacValue;
unsigned char dacStatus;
unsigned char calibrateCount = 0;
long          temperature;
long          code;

float avgTemp;
float avgRaw;
float avgOffset;
float avgGain;
float avgGainCal;
float gainSetting;
unsigned char avgCount = 0;

    WDTCTL = WDTPW | WDTHOLD;   // Stop watchdog timer

    // Configure processor and SMCLK for 1MHz Clock.
    BCSCTL1 = CALBC1_1MHZ;              // BCSCTL1 Calibration Data for 1MHz
    DCOCTL = CALDCO_1MHZ;               // DCOCTL Calibration Data for 1MHz
    BCSCTL2 &= ~SELS;                   // Select DCOCLK as SMCLK source

    Setup_IO ();

    __enable_interrupt();

    // Configure the SPI components
    Setup_SPI_Master (SPI_PORT, ADS_DAC_DOUT, ADS_DAC_DIN, ADS_DAC_CLK,
                      SECONDARY_PERIPHERAL, USCI_B_SIDE, SMCLK_1_MHZ, SPI_1_MHZ,
                      SPI_INACTIVE_LOW_POL, SPI_CHANGE_ON_FIRST_EDGE);

    // Identify which Port and Pins are used for the CS signal for the ADS1220 on the SPI bus
    Setup_ADS1220_CS (ADS_CS_PORT, ADS_CS_N);

    // Configure the UART, used for debug and demo purposes
    Setup_UART(UART_PORT, UART_TX_PIN, UART_RX_PIN, SECONDARY_PERIPHERAL, SMCLK_1_MHZ, UART_9600_BAUD);

    // Reset the ADS1220
    ADS1220_Reset();

    // Determine the ADS1220 Calibration offset - Short the AIN0 and AIN1 together and measure the results 4 times.
    // The average result will be subtracted from all future measurements.
    Setup_ADS1220 (ADS1220_MUX_SHORTED, ADS1220_OP_MODE_NORMAL,
                   ADS1220_CONVERSION_SINGLE_SHOT, ADS1220_DATA_RATE_20SPS, ADS1220_GAIN_16, ADS1220_USE_PGA,
                   ADS1220_IDAC1_AIN3, ADS1220_IDAC2_AIN2, ADS1220_IDAC_CURRENT_250_UA);

    // Delay 500ms to allow for IDAC and Input Filter settling
    __delay_cycles(500000);

    ReadConversionData = 0;
    ADS1220_Start ();             // Kick off conversion

    // Gather and average 8 readings from the ADS1220
    while (calibrateCount < 8)
    {
        while (!ReadConversionData);   // Wait for Data Ready interrupt
        ReadConversionData = 0;
        ADS1220_Get_Conversion_Data ((unsigned char *)tempData);   // Get the raw data
        ADS1220_Offset_Calibrate_Data ((unsigned char *)tempData);        // Send results to calibration function
        calibrateCount++;

        // Start next calibration reading?
        if (calibrateCount < 8)
            ADS1220_Start ();
    }

    // Configure ADS1220 for actual measurements
    Setup_ADS1220 (ADS1220_MUX_AIN0_AIN1, ADS1220_OP_MODE_NORMAL,
                   ADS1220_CONVERSION_CONTINUOUS, ADS1220_DATA_RATE_20SPS, ADS1220_GAIN_16, ADS1220_USE_PGA,
                   ADS1220_IDAC1_AIN3, ADS1220_IDAC2_AIN2, ADS1220_IDAC_CURRENT_250_UA);

    // Configure the DAC161
    Setup_DAC161  (DAC_CS_PORT, DAC_CS_N, DAC161_STD_ERR_CONFIG_MASKED, 3370, 21800);

    Setup_Timer(50000);                  // 50 ms

    StartConversion = 0;
    ReadConversionData = 0;
    UART_Processing = 1;
    DAC_Error_Processing = 0;

    ADS1220_Start ();      // Only one start needed for Continuous Mode

    while (1)
    {
        __bis_SR_register(CPUOFF);

        // In case we extend the time between samples in the future, we will still need to keep the DAC
        // from timing out by sending a NOP command every time the loop is processed.
        DAC161_Nop();

        // Every time the timer interrupt occurs, StartConversion is set and a new conversion is started,
        // when we are in ONE ADS1220_CONVERSION_SINGLE_SHOT Conversion mode. As the ADS1220 is in
        // ADS1220_CONVERSION_CONTINUOUS, this code is commented out.
//        if (StartConversion)
//        {
//            StartConversion = 0;
//            ADS1220_Start ();
//        }

        // Every time the Data Ready interrupt occurs, the Conversion Data is read
        if (ReadConversionData)
        {
            StartConversion = 1;             // Start next conversion immediately
            ADS1220_Get_Conversion_Data_Calibrated ((unsigned char *)tempData);   // Get the temperature data with the calibration value subtracted

            code = (((long)tempData[0] << 16) + ((long)tempData[1] << 8) + (long)tempData[2]);   // take 3 bytes of voltage returned and place in word

            // In case we extend the time between samples in the future, we will still need to keep the DAC
            // from timing out by sending a NOP command every time the loop is processed.
            DAC161_Nop();

            temperature = (long)(interpolateTemperatureValue (code) * 1000.0);    // Get the temperature and convert into millidegrees C

            ReadConversionData = 0;                        // Clear flag so next conversion is read again.

            // Collect the measured values for SAMPLES_PER_DAC_ADJUSTMENT(16) measurements to average
            // Note that everything EXCEPT the temperature measurement and avgOffset is present for display purposes only.
            //  A normal application would only process the temperature measurement.
            if (avgCount == 0)
            {
                // Erase all previous value, as this is the first measurement for this average block
                avgTemp = (float)temperature;
                avgRaw = (float)RawVoltage;
                avgOffset = (float)CalVoltage;
                avgGain = (float)GainVoltage;
                avgGainCal = (float)GainCalibrateValue;
            }
            else
            {
                // Add values from this measurement into the average block
                avgTemp += (float)temperature;
                avgRaw += (float)RawVoltage;
                avgOffset += (float)CalVoltage;
                avgGain += (float)GainVoltage;
                avgGainCal += (float)GainCalibrateValue;

            }
            avgCount ++;

            // If this is the last measurement, calculate the averages and set the system to print
            if (avgCount == SAMPLES_PER_DAC_ADJUSTMENT)
            {
                PrintFlag = 1;
                avgCount = 0;
                temperature = (long)(avgTemp / SAMPLES_PER_DAC_ADJUSTMENT);
                RawVoltage = (long)(avgRaw / SAMPLES_PER_DAC_ADJUSTMENT);
                CalVoltage = (long)(avgOffset / SAMPLES_PER_DAC_ADJUSTMENT);
                GainVoltage = (long)(avgGain / SAMPLES_PER_DAC_ADJUSTMENT);
                GainCalibrateValue = (long)(avgGainCal / SAMPLES_PER_DAC_ADJUSTMENT);

                // Perform Gain Calibration when the user pushes the button on the top of the device.
                // The user MUST have modified the code to place the expected results into the MeasuredGainCodeValue
                //  variable at the top of main.c. The user is expected to place a known resistor value in place of the
                //  RTD and perform gain calibration to determine the system level gain correction. This value will be
                //  stored into flash memory for future use.
                if (GainCal)
                {
                    GainCal = 0;
                    gainSetting = MeasuredGainCodeValue / CalVoltage;
                    write_SegC (gainSetting);
                }

#ifndef DAC_TEST   // Remove for test code

                // Convert the average temperature into a DAC uAmp value  (4000uA = -200 C;   20000uA = +850 C;
                // We will take the measured error of the DAC161 into account by multiplying by a DacErrorCorrection factor.
                // For this application, we have measured a fairly linear error, and therefore determined a single correction
                // factor that we will use for all measurements.
                //  For customers wishing even greater accuracy, the DAC error could be interpolated over multiple points
                //  in the curve, with different correction factors for different areas along the curve.
                uncorrectedDacValue = ConvertTemp2uAmps (temperature);
                dacValueForTemp_uA = (unsigned long)((float)uncorrectedDacValue * DacErrorCorrection);

                // Output value on DAC
                DAC161_Set_Out_Value (dacValueForTemp_uA);
#endif
            }

            if ((UART_Processing) && (PrintFlag))
            {
                P2OUT |=  VCC_ISO_UART_EN;          // Set the isolation for UART
                __delay_cycles(20000);

#ifdef TERMINAL_OUTPUT
                uart_puts ("\r\n-----AVERAGES of 16 Measurements-----\r\n");
                uart_puts ("Temperature in 1000*Degrees C : ");
                if (temperature < 0)
                {
                    uart_puts("-");
                    uart_putLong (temperature * -1);
                }
                else
                {
                    uart_putLong (temperature);
                }

                uart_puts ("\r\n****************\r\n");
                uart_puts ("Raw: ");
                uart_putLong (RawVoltage);

                uart_puts ("    Offset: ");
                uart_putLong (CalVoltage);

                uart_puts ("    Gain: ");
                uart_putLong (GainVoltage);

                uart_puts ("    GainCal: ");
                uart_putLong (GainCalibrateValue);

                uart_puts ("\r\n****************\r\n");
#else
                // Output a Uart Block to GUI out the comport
                uart_putc ('Z');            // Start Byte
                uart_putc (6);              // Length Byte
                uart_putc (1);              // Command Byte
                uart_putc (0);              // Status Byte
                uart_putc ((temperature / 100) >> 8);       // MSB of temperature converted down to 1/10 degree C
                uart_putc ((temperature / 100) & 0xff);     // LSB of temperature converted down to 1/10 degree C
                uart_putc (uncorrectedDacValue >> 8);       // MSB of uAmps
                uart_putc (uncorrectedDacValue & 0xff);     // LSB of uAmps

#endif
            }

#ifdef DAC_TEST   // Test code

            // This block is used to test the DAC. Every time the button on the bottom of the device is pressed,
            // the output of the DAC is increased by 2mA, between 4 and 20mA.
            if (DacIncrement)
            {
                float dacValue;

                DacIncrement = 0;
                // Test code - Ramp from 6mA to 18mA and start over again at 6mA.
                dacValueForTemp_uA += 2000;
                if (dacValueForTemp_uA > 20000)
                    dacValueForTemp_uA = 4000;

                // Apply the DacErrorCorrection to the desired value. If this is measuring
                dacValue = (float)dacValueForTemp_uA * DacErrorCorrection;
                DAC161_Set_Out_Value ((unsigned long)dacValue);
#endif

            // Print out the DAC measurements, both before and after Dac Gain Correction
            if ((UART_Processing) && (PrintFlag))
            {
                PrintFlag = 0;
                uart_puts ("\r\nDAC Value in uA : ");
                uart_putLong (dacValueForTemp_uA);
                uart_puts ("\r\nUncorrected DAC Value in uA : ");

                uart_putLong (uncorrectedDacValue);
                dacStatus = DAC161_Read_Status();
                uart_puts ("      DAC Status : ");
                uartPutHex (dacStatus);
                uart_puts ("\r\n");
                P2OUT &=  ~VCC_ISO_UART_EN;          // Clear the isolation for UART
            }

#ifdef DAC_TEST   // Test code
            }
#endif
        }

        // Did the DAC161 indicate an error?
        if (DAC_Error_Processing)
        {
            // Perform DAC ERROR Processing Here
            DAC_Error_Processing = 0;
            dacStatus = DAC161_Read_Status();         // Read Status to clear error

            if (UART_Processing)
            {
                P2OUT |=  VCC_ISO_UART_EN;          // Set the isolation for UART
                __delay_cycles(20000);
                uart_puts ("\r\nDAC Error! ");
                uart_puts ("      DAC Status : ");
                uartPutHex (dacStatus);
                uart_puts ("\r\n\n\n");
                P2OUT &=  ~VCC_ISO_UART_EN;          // Clear the isolation for UART
            }
        }

    }

}


/**************************************************************************************************************************************************
* TIMER0_A1_ISR
**************************************************************************************************************************************************/
/*!
* @brief Timer Interrupt Routine
*
*  Start the Conversion measurement based on timer event
*
* @return None
**************************************************************************************************************************************************/
#pragma vector = TIMER0_A1_VECTOR
__interrupt void TIMER0_A1_ISR (void)
{

    TA0CTL &= ~TAIFG;

    // Used to slow down measurements to once every timer interval when the ADS1220 is in ADS1220_CONVERSION_SINGLE_SHOT mode
//    StartConversion = 1;

    __bic_SR_register_on_exit(CPUOFF);      // Exit LPM0

}

/**************************************************************************************************************************************************
* PORT1_ISR
**************************************************************************************************************************************************/
/*!
* @brief Port 1 Interrupt Routine
*
* This interrupt will occur when the system button is pushed. This button will be used to start/stop UART output of measurement data.
*
* @return None
**************************************************************************************************************************************************/
#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR (void)
{
unsigned char interrupts = P1IFG;

    P1IFG = 0;

    // The user button will turn on/off the UART outputs
    if (interrupts & USR_USER_INTERRUPT_N)
    {
#ifndef DAC_TEST   // NOT DAC Test code
        UART_Processing ^= 1;                // This is the normal use of the button to allow the UART to be enabled/disabled
#else              // DAC Test code - Use user button to increment the DAC output level
        DacIncrement = 1;                 // During testing, we will use the button to increment dac processing. Default OFF
#endif
    }

}

/**************************************************************************************************************************************************
* Port2_ISR
**************************************************************************************************************************************************/
/*!
* This interrupt will occur when a signal from the ADS1220 indicates data is ready, or a signal from the DAC161 indicates an error has occurred.
*
* @brief Port 2 Interrupt Service Routine
*
* @return None
**************************************************************************************************************************************************/
#pragma vector = PORT2_VECTOR
__interrupt void PORT2_ISR (void)
{
unsigned char interrupts = P2IFG;

    P2IFG = 0;

    // Has a Data Ready occurred from the ADS1220?
    if (interrupts & ADS_DRDY_N)
    {
        ReadConversionData = 1;
    }

    // Has an error from the DAC occurred?
    if (interrupts & DAC_ERROR_N)
    {
        DAC_Error_Processing = 1;
    }

    if (interrupts & USR_GAIN_CAL)
    {
        // If there is no GainCalibration in place, allow the system to perform GainCalibration
        if (*(unsigned long *)&FlashGainCorrection == 0xffffffff)
            GainCal = 1;
    }

    __bic_SR_register_on_exit(CPUOFF);      // Exit LPM0

}
